#ifndef HARKIO_HARK_HARKFILEBINARYREADER_HPP_
#define HARKIO_HARK_HARKFILEBINARYREADER_HPP_

#include <istream>
#include <boost/shared_ptr.hpp>
#include <harkio2/AnalyzeError.hpp>
#include <harkio2/hark/HarkFileReader.hpp>

/** @file HarkFileBinaryReader.hpp
 * @brief HarkFileBinaryReaderクラスの定義
 */

namespace harkio2
{
namespace hark
{

class HarkFile;

/** @brief バイナリ形式のHARKファイルに対応した HarkFileReader の実装。
 */
class HarkFileBinaryReader : public HarkFileReader
{
public:
    /** @brief バイナリリーダーを構築する。
     *
     * データ源はstd::istream（のサブクラス）であり、引数'in'で指定される。
     * 入力ストリームはシークをサポートする型であり、
     * さらにバイナリモードで開かれている必要がある。<br>
     * 典型的にはstd::ifstreamのコンストラクタにstd::ios::binaryを指定することで、
     * この要求を満たすことができる。
     * @param in データ読み込み元の入力ストリーム
     */
    HarkFileBinaryReader(std::istream& in);

    /** @brief HarkFile データ構造をコンストラクタで指定されたストリームから読み込み、返す。
     * @return 読み込んだデータ
     */
    virtual boost::shared_ptr<harkio2::hark::HarkFile> Read() throw (harkio2::AnalyzeError);

#ifdef HARKIO_UNITTEST
public:
#else
private:
#endif
    /// @brief Helper class.
    class Helper;
private:
    /// @brief Data source.
    std::istream& in;
};

}
}

#endif // HARKIO_HARK_HARKFILEBINARYREADER_HPP_

