#ifndef HARKIO_HARKCONFIG_HARKCONFIGFILETEXTHELPER_HPP_
#define HARKIO_HARKCONFIG_HARKCONFIGFILETEXTHELPER_HPP_

#include <ostream>
#include <string>
#include <boost/shared_ptr.hpp>
#include <harkio2/AnalyzeError.hpp>

/** @file HarkConfigFileTextHelper.hpp
 * @brief HarkConfigFileTextHelper class definition.
 */

namespace harkio2
{
namespace harkconfig
{

class HarkConfigFile;

/** @brief Helper class interface for HarkConfigFileTextReader.
 * This interface helps HarkConfigFileTextReader to read lines and construct HarkConfigFile data structure.
 */
class HarkConfigFileTextHelper
{
public:
    /// @brief Virtual destructor for derivation.
    virtual ~HarkConfigFileTextHelper() {}

    /** @brief Returns a constructed HarkConfigFile structure.
     * Returns a HarkConfigFile structure constructed by calling AnalyzeLine() till now.
     * If there is no AnalyzeLine() calling, returns empty pointer.
     * @return A constructed HarkConfigFile.
     */
    virtual boost::shared_ptr<HarkConfigFile> GetData() = 0;

    /** @brief Inputs a line and updates a inside HarkConfigFile structure.
     * Inputs a line which was read from a file, and updates a inside HarkConfigFile structure.
     * @param line A line which was read from a file
     * @param lineNum The line number of 'line' (0:unknown line number)
     */
    virtual void AnalyzeLine(const std::string& line, unsigned int lineNum = 0) = 0;

    /** @brief Writes a inside HarkConfigFile to a output stream.
     * @param out A output stream to write data.
     */
    virtual void Write(std::ostream& out) = 0;

};

}
}

#endif // HARKIO_HARKCONFIG_HARKCONFIGFILETEXTHELPER_HPP_

