#define BOOST_TEST_MODULE "testcase for DataSet class and sub classes"
#define BOOST_TEST_DYN_LINK
#include "common.hpp"

#include <boost/test/unit_test.hpp>

#include <vector>
#include <boost/shared_ptr.hpp>
#include <harkio2/hark/DataSet.hpp>
#include <harkio2/NotInitialized.hpp>
#include <harkio2/hark/hgtf/Fmt.hpp>
#include <harkio2/hark/hgtf/Spec.hpp>
#include <harkio2/hark/hgtf/M2PG.hpp>
#include <harkio2/hark/hgtf/GTF.hpp>
#include <harkio2/hark/hgtf/SM.hpp>

using namespace boost;
using namespace std;

BOOST_AUTO_TEST_CASE(fmt_test)
{
    harkio2::hark::hgtf::Fmt1_0 fmt10;

    BOOST_CHECK_THROW(fmt10.GetFormatId(), harkio2::NotInitialized);

    fmt10.SetFormatId(1);
    fmt10.SetNMic(2);
    fmt10.SetNFFT(256);
    fmt10.SetNImpulseResponse(512);
    fmt10.SetFS(16000);

    BOOST_CHECK_EQUAL(fmt10.GetFormatId(), 1);
    BOOST_CHECK_EQUAL(fmt10.GetFS(), 16000);
}

BOOST_AUTO_TEST_CASE(spec_test)
{
    harkio2::hark::hgtf::Spec1_0 spec10;

    BOOST_CHECK_THROW(spec10.GetDate(), harkio2::NotInitialized);

    spec10.SetDate(u8"2011/07/11");
    spec10.SetLocation(u8"smart room");

    BOOST_CHECK_EQUAL(spec10.GetDate(), u8"2011/07/11");
    BOOST_CHECK_EQUAL(spec10.GetLocation(), u8"smart room");
}

BOOST_AUTO_TEST_CASE(m2pg_test)
{
    harkio2::hark::hgtf::M2PG1_0 m2pg10;

    BOOST_CHECK_THROW(m2pg10.GetND(), harkio2::NotInitialized);
    BOOST_CHECK_THROW(m2pg10.GetSourcePositions(), harkio2::NotInitialized);

    vector<vector<harkio2::Vector3<float> > > positions(
            2, vector<harkio2::Vector3<float> >(1));
    positions[0][0] = harkio2::Vector3<float>(0, 0, 0);
    positions[1][0] = harkio2::Vector3<float>(1, 0, 0);

    m2pg10.SetND(2);
    m2pg10.SetNR(1);
    m2pg10.SetSourcePositions(positions);

    BOOST_CHECK_EQUAL(m2pg10.GetND(), 2);
    BOOST_CHECK_EQUAL(m2pg10.GetNR(), 1);
    BOOST_REQUIRE_EQUAL(m2pg10.GetSourcePositions().size(), 2);
    BOOST_REQUIRE_EQUAL(m2pg10.GetSourcePositions()[0].size(), 1);
    BOOST_REQUIRE_EQUAL(m2pg10.GetSourcePositions()[1].size(), 1);
    BOOST_CHECK(m2pg10.GetSourcePositions()[0][0] == harkio2::Vector3<float>(0, 0, 0));
    BOOST_CHECK(m2pg10.GetSourcePositions()[1][0] == harkio2::Vector3<float>(1, 0, 0));
}

BOOST_AUTO_TEST_CASE(gtf_test)
{
    harkio2::hark::hgtf::GTF1_0 gtf10;

    BOOST_CHECK_THROW(gtf10.GetNSource(), harkio2::NotInitialized);

    vector<harkio2::Vector3<float> > positions(2);
    positions[0] = harkio2::Vector3<float>(0, 2, 0);
    positions[1] = harkio2::Vector3<float>(1, 2, 0);

    gtf10.SetNSource(2);
    gtf10.SetSourcePositions(positions);

    BOOST_CHECK_EQUAL(gtf10.GetNSource(), 2);
    BOOST_REQUIRE_EQUAL(gtf10.GetSourcePositions().size(), 2);
    BOOST_CHECK(gtf10.GetSourcePositions()[0] == harkio2::Vector3<float>(0, 2, 0));
    BOOST_CHECK(gtf10.GetSourcePositions()[1] == harkio2::Vector3<float>(1, 2, 0));
}

BOOST_AUTO_TEST_CASE(sm_test)
{
    harkio2::hark::hgtf::SM1_0 sm10;

    BOOST_CHECK_THROW(sm10.GetNSource(), harkio2::NotInitialized);

    vector<harkio2::Vector3<float> > positions(2);
    positions[0] = harkio2::Vector3<float>(0, 4, 0);
    positions[1] = harkio2::Vector3<float>(1, 4, 0);

    sm10.SetNSource(2);
    sm10.SetSourcePositions(positions);

    BOOST_CHECK_EQUAL(sm10.GetNSource(), 2);
    BOOST_REQUIRE_EQUAL(sm10.GetSourcePositions().size(), 2);
    BOOST_CHECK(sm10.GetSourcePositions()[0] == harkio2::Vector3<float>(0, 4, 0));
    BOOST_CHECK(sm10.GetSourcePositions()[1] == harkio2::Vector3<float>(1, 4, 0));
}

