#define BOOST_TEST_MODULE "testcase for HarkFileBinaryWriter class"
#define BOOST_TEST_DYN_LINK
#include "common.hpp"

#include <boost/test/unit_test.hpp>

#include <fstream>
#include <harkio2/hark/hgtf/Fmt.hpp>
#include <harkio2/hark/hgtf/Spec.hpp>
#include <harkio2/hark/hgtf/GTF.hpp>
#include <harkio2/hark/HarkFile.hpp>
#include <harkio2/hark/HarkFileBinaryReader.hpp>
#include <harkio2/hark/HarkFileBinaryWriter.hpp>
#include <harkio2/hark/hgtf/Fmt.hpp>

BOOST_AUTO_TEST_CASE(read_test)
{
    typedef std::complex<float> C;

    std::stringstream harkFileData;

    boost::shared_ptr<harkio2::hark::HarkFileWriter> writer(
            new harkio2::hark::HarkFileBinaryWriter(harkFileData));

    // make Fmt dataset
    std::vector<harkio2::Vector3<float>> micpos(2, harkio2::Vector3<float>(10, 20, 30));
    boost::shared_ptr<harkio2::hark::hgtf::Fmt1_0> fmt10(new harkio2::hark::hgtf::Fmt1_0());
    fmt10->SetFormatId(2);
    fmt10->SetNMic(2);
    fmt10->SetNFFT(1024);
    fmt10->SetNImpulseResponse(2048);
    fmt10->SetFS(16000);
    fmt10->SetMicPositions(micpos);

    // make GTF dataset
    std::vector<harkio2::Vector3<float>> srcpos(3, harkio2::Vector3<float>(40, 50, 60));
    std::vector<unsigned int> table(512);
    std::vector<std::vector<std::vector<C>>> tf(
            3, std::vector<std::vector<C>>(
                fmt10->GetNMic(), std::vector<C>(
                    512, C(60, 70))));
    for (size_t f = 0; f < table.size(); ++f) table[f] = f;
    boost::shared_ptr<harkio2::hark::hgtf::GTF1_0> gtf10(new harkio2::hark::hgtf::GTF1_0());
    gtf10->SetNSource(3);
    gtf10->SetSourcePositions(srcpos);
    gtf10->SetNFIndex(512);
    gtf10->SetFIndexToBinIndexTable(table);
    gtf10->SetTFs(tf);

    // make HARK file
    boost::shared_ptr<harkio2::hark::HarkFile> hark(new harkio2::hark::HarkFile());
    hark->AddDataSet(fmt10);
    hark->AddDataSet(gtf10);

    // write to file
    writer->Write(hark);

    // check if data was successfully written
    boost::shared_ptr<harkio2::hark::HarkFileReader> reader(
            new harkio2::hark::HarkFileBinaryReader(harkFileData));
    boost::shared_ptr<harkio2::hark::HarkFile> rhark = reader->Read();

    BOOST_REQUIRE(rhark);

    boost::shared_ptr<harkio2::hark::DataSet> rfmt = rhark->GetDataSet("hgtf::Fmt");
    boost::shared_ptr<harkio2::hark::hgtf::Fmt1_0> rfmt10 = boost::dynamic_pointer_cast<harkio2::hark::hgtf::Fmt1_0>(rfmt);

    BOOST_REQUIRE(rfmt10);
    BOOST_CHECK_EQUAL(rfmt10->GetFormatId(), 2);
    BOOST_CHECK_EQUAL(rfmt10->GetNMic(), 2);
    BOOST_CHECK_EQUAL(rfmt10->GetFS(), 16000);

    boost::shared_ptr<harkio2::hark::DataSet> rgtf = rhark->GetDataSet("hgtf::GTF");
    boost::shared_ptr<harkio2::hark::hgtf::GTF1_0> rgtf10 = boost::dynamic_pointer_cast<harkio2::hark::hgtf::GTF1_0>(rgtf);

    BOOST_REQUIRE(rgtf10);
    BOOST_CHECK_EQUAL(rgtf10->GetNSource(), 3);
    BOOST_REQUIRE_EQUAL(rgtf10->GetTFs().size(), 3);
    BOOST_REQUIRE_EQUAL(rgtf10->GetTFs()[0].size(), 2);
    BOOST_REQUIRE_EQUAL(rgtf10->GetTFs()[0][0].size(), 512);
    BOOST_CHECK_EQUAL(rgtf10->GetTFs()[0][0][0], C(60, 70));
}

